
#include "Direct3D.h"

#include <dxerr9.h>



CDirect3D::CDirect3D()
{
    m_lpD3D        = NULL;
    m_lpD3DFont    = NULL;
    m_lpD3DDevice  = NULL;
    m_lpBackBuffer = NULL;
}

CDirect3D::~CDirect3D()
{
    CleanUp();
}

// Direct3D initialisieren
BOOL CDirect3D::Init(HWND hWnd, BOOL bWindowed)
{
    // Direct3D-Objekt erzeugen
    m_lpD3D = Direct3DCreate9(D3D_SDK_VERSION);

    if(NULL == m_lpD3D)
    { 
        // Fehler, D3D-Objekt wurde nicht erzeugt
        return FALSE;
    }

    // Parameter fr den Modus festlegen
    D3DPRESENT_PARAMETERS PParams;
    ZeroMemory(&PParams,sizeof(PParams));

    PParams.SwapEffect       = D3DSWAPEFFECT_DISCARD;
    PParams.hDeviceWindow    = hWnd;
    PParams.Windowed         = bWindowed;

    PParams.BackBufferWidth  = SCR_WIDTH;
    PParams.BackBufferHeight = SCR_HEIGHT;
    PParams.BackBufferFormat = D3DFMT_A8R8G8B8;

    HRESULT hr;
    
    // Direct3D-Gert anlegen
    if(FAILED(hr = m_lpD3D->CreateDevice(
                                 D3DADAPTER_DEFAULT,
                                 D3DDEVTYPE_HAL,
                                 hWnd,
                                 D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                 &PParams,
                                 &m_lpD3DDevice)))
    {
        // Fehler, Gert kann nicht angelegt werden
        const char* Err = DXGetErrorDescription9(hr);
 
        DXTRACE_MSG(Err);

        return FALSE;
    }

    // Schrift erzeugen
    CreateFont();
 
    // Zeiger auf dem Hintergrundpuffer holen
    m_lpD3DDevice->GetBackBuffer(0,0,
                                 D3DBACKBUFFER_TYPE_MONO,
                                 &m_lpBackBuffer);

    return TRUE;
}

// Zeiger auf den Hintergrundbuffer zurckgeben
LPDIRECT3DSURFACE9 CDirect3D::GetBackbuffer(void)
{
    return m_lpBackBuffer;
}

// Farbe fr den Backbuffer festlegen
void CDirect3D::SetClearColor(D3DCOLOR Color)
{
    m_ClearColor = Color;
}

// Text an Position x,y ausgeben
void CDirect3D::DrawText(LPCSTR Text, int x, int y, D3DCOLOR TextColor)
{
    RECT r = { x, y, 0, 0 };

    // Gre des Rechtecks berechnen
    m_lpD3DFont->DrawText(Text, -1, &r, DT_CALCRECT, TextColor);

    // Text ausgeben
    m_lpD3DFont->DrawText(Text, -1, &r, DT_CENTER, TextColor);  
}

// Methoden zum Start/Beenden der Szene 
void CDirect3D::BeginScene(void)
{
    m_lpD3DDevice->Clear(0,0,D3DCLEAR_TARGET,m_ClearColor,0,0);
    
    m_lpD3DDevice->BeginScene();
}


void CDirect3D::EndScene(void)
{
    m_lpD3DDevice->EndScene();

    m_lpD3DDevice->Present(0,0,0,0);
}
 
// Erzeugen der D3D-Schrift
void CDirect3D::CreateFont(void)
{
    // Struktur fr die Beschreibung der Schriftart
    // anlegen und Elemente mit 0 initialisieren
    LOGFONT    LogFont = { 0 };

    // das Element lfFaceName muss den Namen der 
    // gewnschten Schriftart enthalten
    strcpy(LogFont.lfFaceName, "Arial");

    // nach erfolgreichem Aufruf zeigt lpD3DFont 
    // auf ein Objekt vom Typ D3DXFont
    D3DXCreateFontIndirect(m_lpD3DDevice,
                           &LogFont,
                           &m_lpD3DFont);
}


// Diese Methode zeichnet eine Ellipse
void CDirect3D::DrawEllipse(int x1,int y1, int x2, int y2, LPDIRECT3DSURFACE9 lpSurface)
{
    HDC hDC;

    if(FAILED(lpSurface->GetDC(&hDC)))
    {
        DXTRACE_MSG("DC ist nicht verfgbar");
    }
    else
    {
        // Ellipse zeichnen
        Ellipse(hDC,x1,y1,x2,y2);

        // Wichtig! Device Context wieder freigeben
        lpSurface->ReleaseDC(hDC);
    }
}


// Screenshot erzeugen
void CDirect3D::CreateScreenshot(char* Filename)
{
    // temporre Oberflche fr den Screenshot anlegen
    LPDIRECT3DSURFACE9 lpSurface;

    m_lpD3DDevice->CreateOffscreenPlainSurface(
                                  1024,768,
                                  D3DFMT_A8R8G8B8,
                                  D3DPOOL_SYSTEMMEM,
                                  &lpSurface,0);
    
    // Inhalt des Frontbuffer in die Oberflche kopieren
    m_lpD3DDevice->GetFrontBufferData(0,lpSurface);
    
    // Oberflche in Datei speichern
    D3DXSaveSurfaceToFile(Filename,D3DXIFF_BMP,
                          lpSurface,0,NULL);

    // temporre Oberflche wieder freigeben
    lpSurface->Release();
}

// das Direct3D-Device zurckgeben
LPDIRECT3DDEVICE9 CDirect3D::GetDevice(void)
{
    return m_lpD3DDevice;
}


// Freigeben der Objekte
void CDirect3D::CleanUp(void)
{
    // testen, ob die Objekt angelegt 
    // wurden und ggf. freigeben
        
    if(NULL != m_lpD3DFont)
    {
        m_lpD3DFont->Release();
        m_lpD3DFont = NULL;
    }
  
    if(NULL != m_lpBackBuffer)
    {
        m_lpBackBuffer->Release();
        m_lpBackBuffer = NULL;
    }    

    if(NULL != m_lpD3DDevice)
    {
        m_lpD3DDevice->Release();
        m_lpD3DDevice = NULL;
    }
    
    if(NULL != m_lpD3D)
    {
        m_lpD3D->Release();
        m_lpD3D = NULL;
    }
} 