/*
    Jetzt lerne ich Spieleprogrammierung mit DirectX und Visual C++
       
    Programmname: SpriteBT

    Beschreibung: Dieses Programm zeigt, wie Sie Sprites animieren knnen.
	              In diesem Beispiel werden die Sprites aus einer einzigen
				  Textur geladen.
    
                  Fhren Sie das Programm durch Drcken von F5 aus.
*/

// Headerdateien
#include "Direct3D.h"
#include "Sprite.h"

// Anwendungsfenster erzeugen
HWND CreateMainWindow(HINSTANCE hInstance);

// Callback Funktion zur Nachrichtenbehandlung
LRESULT CALLBACK MessageHandler(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

// Das Fensterhandle
HWND hWnd = 0;

// Instanz der Klasse CDirect3D
CDirect3D Direct3D;

// Instanz der Klasse CSprite
CSprite Sprite;

// Windows main-Funktion
int WINAPI WinMain(HINSTANCE hInstance,      
                   HINSTANCE hPrevInstance,  
                   LPSTR lpCmdLine,          
                   int nCmdShow)             
{   
    MessageBox(0,"Dieses Beispiel zeigt, wie ein Sprite animiert wird.\n\n"
                 "Die Spritegrafik befindet sich in diesem Beispiel in einer Textur."
				 "Bewegung des Sprites mit den Pfeiltasten\n"
                 "Rotation des Sprites mit Space\n"
                 "Skalierung des Sprites mit den Control/Shift-Tasten", "SpriteAnim",MB_OK);
    
    
    // Fenster erzeugen und Handle speichern
    hWnd = CreateMainWindow(hInstance);

    // Wenn der Rckgabewert 0 ist, ist ein Fehler aufgetreten
    if(0 == hWnd)
    {
        MessageBox(0,"Fenster konnte nicht erzeugt werden","Fehler",MB_OK);
        return -1;
    }

    // Direct3D initialisieren
    if(!Direct3D.Init(hWnd,FALSE))
    {
        return -1;
    }

    // Maus-Cursor verstecken
    ShowCursor(FALSE);

    // Farbe die beim Lschen verwendet werden soll
    Direct3D.SetClearColor(D3DCOLOR_XRGB(0,0,0));
  
    // Sprite anlegen - Create Methode wurde 
    Sprite.Create(Direct3D.GetDevice(),2,71,120);
    
    // Grafiken (Texturen) laden
    char Filename[128];

    // Dateiname der Textur mit den Spritegrafiken
	sprintf(Filename,"walkexample.png");

    Sprite.AddTexture(Filename);
    
    // Struktur, in der Informationen zur Nachricht gespeichert werden
    MSG msg = { 0 };
    
    while(msg.message != WM_QUIT)
    {
        if(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else
        {
            Direct3D.BeginScene();
           
            // Tasten abfragen und Sprite entsprechend bewegen
            if(GetAsyncKeyState(VK_LEFT) & 0x8000)
            {
                Sprite.Move(LEFT);
            }
            else if(GetAsyncKeyState(VK_RIGHT) & 0x8000)
            {
                Sprite.Move(RIGHT);
            }

            if(GetAsyncKeyState(VK_UP) & 0x8000)
            {
                Sprite.Move(UP);
            }
            else if(GetAsyncKeyState(VK_DOWN) & 0x8000)
            {
                Sprite.Move(DOWN);
            }
      
            // Wenn Space gedrckt wurde, Sprite rotieren
            if(GetAsyncKeyState(VK_SPACE) & 0x8000)
            {            
                Sprite.Rotate(.1f);
            }

            // Bei Shift-Taste Sprite vergrern
            if(GetAsyncKeyState(VK_SHIFT) & 0x8000)
            {            
                Sprite.Scale(.1f);
            }
            // Bei Control-Taste Sprite verkleinern
            if(GetAsyncKeyState(VK_CONTROL) & 0x8000)
            {            
                Sprite.Scale(-.1f);
            }
     
            // Zeichnen des Sprites
            Sprite.Draw();

            Direct3D.EndScene();
        }
    }
    
    // Rckgabewert an Windows
    return 0;
}



// Fenster erzeugen
HWND CreateMainWindow(HINSTANCE hInstance)
{
    WNDCLASSEX wndClass = 
    { 
        sizeof(WNDCLASSEX),                                 // Gre angeben
        CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW,    // Standardstile        
        MessageHandler,                                     // Callback-Funktion
        0,                                                  // Zustzliche Angaben
        0,                                                  // nicht bentigt
        hInstance,                                          // Anwendungsinstanz
        LoadIcon(NULL, IDI_WINLOGO),                        // Windows-Logo
        LoadCursor(NULL, IDC_ARROW),                        // Normaler Cursor
        (HBRUSH)GetStockObject(WHITE_BRUSH),                // Weier Pinsel
        NULL,                                               // kein Men
        "WindowClass",                                      // Der Name der Klasse
        LoadIcon(NULL, IDI_WINLOGO)                         // Windows Logo
    };

    // Klasse registrieren
    RegisterClassEx(&wndClass);

    return CreateWindowEx(NULL,                   // Keine erweiterten Stile nutzen
                          "WindowClass",          // Klassenname
                          "SpriteAnim",           // Fenstertitel
                          WS_OVERLAPPEDWINDOW |   // Fenster
                          WS_VISIBLE,             // Eigenschaften
                          0, 0,                   // Anfangsposition 
                          SCR_WIDTH, SCR_HEIGHT,  // und Gre des Fensters
                          NULL,                   // Handle des Elternfensters
                          NULL,                   // Handle des Mens
                          hInstance,              // Anwendungsinstanz
                          NULL);                  // wird nicht bentigt
}


// Diese Funktion wird von Windows aufgrufen, wenn
// eine Nachricht fr Ihr Programm vorliegt
LRESULT CALLBACK MessageHandler(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    // testen, um welche Nachricht es sich handelt
    switch(msg)
    {
     // Programm beenden, wenn das Fenster 
     // geschlossen wird 
    case WM_DESTROY:
            PostQuitMessage(0);
            return 0;
         break;

    case WM_KEYDOWN:
           switch(wParam)
           {
             // Bei F1 einen Screenshot speichern
             case VK_F1:   
                    Direct3D.CreateScreenshot("c:\\screenshot.bmp");
                 break;
     
             // Wenn ESC gedrckt, Programm beenden
             case VK_ESCAPE:
                     DestroyWindow(hWnd);
                  break;                      
           }
         break;
    }
    // Standardnachrichtenverarbeitung von Windows
    return DefWindowProc(hwnd, msg, wParam, lParam);
}
