#include "Sprite.h"

// Konstruktor - Initialisierungen
CSprite::CSprite(void)
{
    m_Rotation = 0;

    m_StepX    = 0;
    m_StepY    = 0;

    m_Width    = 0;
    m_Height   = 0;

    m_AnimationSpeed = 0;
    m_AnimationDelay = 0;
    m_CurrentImageX  = 0;
	m_CurrentImageY  = 0;

    m_lpDevice = NULL;                  
    m_lpSprite = NULL;
}

// Destructor - freigeben der Objekte
CSprite::~CSprite(void)
{
    if(NULL != m_lpSprite)
    {
        m_lpSprite->Release();
        m_lpSprite = NULL;
    }

    // alle Texturen im vector freigeben
    for(int i=0;i<m_AnimationVec.size();i++)
    {
        m_AnimationVec[i]->Release();
    }
}

// Position setzen
void CSprite::SetPosition(float x,float y)
{
    m_vPosition.x = x;
    m_vPosition.y = y;
}

// Rotation setzen
void CSprite::SetRotation(float rotation)
{
    m_Rotation = rotation;
}

// Skalierungsfaktor setzen
void CSprite::SetScale(float scale)
{
    m_vScale.x = scale;
    m_vScale.y = scale;
}

// Schrittweite bestimmen
void CSprite::SetStep(float StepX,float StepY)
{
    m_StepX = StepX;
    m_StepY = StepY;
}

// Sprite kreiern
void CSprite::Create(LPDIRECT3DDEVICE9 lpDevice, int AnimationSpeed, int Width, int Height)
{
    m_lpDevice = lpDevice;
    
    m_AnimationSpeed = AnimationSpeed;
    
    D3DXCreateSprite(m_lpDevice, &m_lpSprite);
    
    m_Width  = Width;
    m_Height = Height;

    m_vRotationCenter = D3DXVECTOR2((float)m_Width/2, (float)m_Height/2);
    
    m_vScale = D3DXVECTOR2(1.0f,1.0f);

    m_StepX = 10.0f;
    m_StepY = 10.0f;
}

// Textur fr das Sprite festlegen
void CSprite::AddTexture(LPCSTR lpFileName)
{
    D3DXIMAGE_INFO ImgInfo;
    D3DXGetImageInfoFromFile(lpFileName,&ImgInfo);
    
    m_vRotationCenter.x = m_Width  / 2.0f;
    m_vRotationCenter.y = m_Height / 2.0f;

	// Breite und Hhe der Textur speichern
	m_nTextureWidth  = ImgInfo.Width;
	m_nTextureHeight = ImgInfo.Height;
   
    LPDIRECT3DTEXTURE9 lpTexture;
    
    D3DXCreateTextureFromFileEx(m_lpDevice,
                                lpFileName,
                                ImgInfo.Width,
                                ImgInfo.Height,
                                1,0,
                                D3DFMT_UNKNOWN,
                                D3DPOOL_MANAGED,
                                D3DX_FILTER_NONE,
                                D3DX_FILTER_NONE,
                                0,0,0,&lpTexture);

    // Zeiger auf die Textur im Vector speichern
    m_AnimationVec.push_back(lpTexture);
}

// Bewegung des Sprite
void CSprite::Move(DIRECTION dir)
{
    // Bei der Bewegung werden die Bildschirmbegrenzungen beachtet
    switch(dir)
    {
        case LEFT:
                if(m_vPosition.x > 0)
                {
                    m_vPosition.x -= m_StepX;
                }
        break;

        case RIGHT:
                if(m_vPosition.x < SCR_WIDTH - m_Width)
                {
                    m_vPosition.x += m_StepX;
                }
        break;

        case UP:
                if(m_vPosition.y > 0)
                {
                    m_vPosition.y -= m_StepY;
                }
        break;

        case DOWN:
                if(m_vPosition.y < SCR_HEIGHT - m_Height)
                {
                    m_vPosition.y += m_StepY;
                }
        break;
    }
}

// Skalierung des Sprites
void CSprite::Scale(float dScale)
{
    m_vScale.x += dScale;
    m_vScale.y += dScale;
}

// Rotation des Sprites
void CSprite::Rotate(float dRotation)
{
    m_Rotation += dRotation;
}

// Sprite zeichnen
void CSprite::Draw(void)
{
	// Rechteck fr 
	RECT SpriteRect;
    
	// testen, ob die nchste Animation angezeigt werden soll
    if(++m_AnimationDelay > m_AnimationSpeed)
    {
        m_AnimationDelay = 0;

        // wenn ja, berechnen, welcher Ausschnitt verwendet werden soll.
        if(++m_CurrentImageX >= m_nTextureWidth / m_Width)
		{
			m_CurrentImageY++;
			m_CurrentImageX = 0;
			
			if(m_CurrentImageY >= m_nTextureHeight / m_Height)
			{
				m_CurrentImageY = 0;
				m_CurrentImageX = 0;
			}
		}		
    }

	// das Recteck fr die Spritegrafik in der Textur berechnen
	SpriteRect.left   = m_CurrentImageX * m_Width;
	SpriteRect.right  = SpriteRect.left + m_Width;

	SpriteRect.top    = m_CurrentImageY * m_Height;
	SpriteRect.bottom = SpriteRect.top + m_Height;

    // Sprite (Texturausschnitt) in der passenden Animationsstufe zeichnen
    m_lpSprite->Draw(m_AnimationVec[0],
                     &SpriteRect,
                     &m_vScale,
                     &m_vRotationCenter,
                     m_Rotation,
                     &m_vPosition,
                     0xFFFFFFFF);
}
